@@ -15,6 +15,7 @@
 #include "qcocoahelpers.h"
 
 #include <QtCore/qfileinfo.h>
+#include <QtCore/private/qcore_mac_p.h>
 #include <QtGui/private/qfont_p.h>
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtGui/private/qcoregraphics_p.h>
@@ -30,6 +31,7 @@
 #include "qcocoacolordialoghelper.h"
 #include "qcocoafiledialoghelper.h"
 #include "qcocoafontdialoghelper.h"
+#include "qcocoamessagedialog.h"
 
 #include <CoreServices/CoreServices.h>
 
@@ -92,6 +94,9 @@ static QPalette *qt_mac_createSystemPalette()
     palette->setColor(QPalette::Inactive, QPalette::PlaceholderText, qc);
     palette->setColor(QPalette::Disabled, QPalette::PlaceholderText, qc);
 
+    qc = qt_mac_toQColor([NSColor controlAccentColor]);
+    palette->setColor(QPalette::Accent, qc);
+
     return palette;
 }
 
@@ -219,6 +224,8 @@ QCocoaTheme::QCocoaTheme()
         NSSystemColorsDidChangeNotification, [this] {
             handleSystemThemeChange();
     });
+
+    updateColorScheme();
 }
 
 QCocoaTheme::~QCocoaTheme()
@@ -237,6 +244,9 @@ void QCocoaTheme::reset()
 void QCocoaTheme::handleSystemThemeChange()
 {
     reset();
+
+    updateColorScheme();
+
     m_systemPalette = qt_mac_createSystemPalette();
     m_palettes = qt_mac_createRolePalettes();
 
@@ -250,13 +260,15 @@ void QCocoaTheme::handleSystemThemeChange()
 
 bool QCocoaTheme::usePlatformNativeDialog(DialogType dialogType) const
 {
-    if (dialogType == QPlatformTheme::FileDialog)
+    switch (dialogType) {
+    case QPlatformTheme::FileDialog:
+    case QPlatformTheme::ColorDialog:
+    case QPlatformTheme::FontDialog:
+    case QPlatformTheme::MessageDialog:
         return true;
-    if (dialogType == QPlatformTheme::ColorDialog)
-        return true;
-    if (dialogType == QPlatformTheme::FontDialog)
-        return true;
-    return false;
+    default:
+        return false;
+    }
 }
 
 QPlatformDialogHelper *QCocoaTheme::createPlatformDialogHelper(DialogType dialogType) const
@@ -268,6 +280,8 @@ QPlatformDialogHelper *QCocoaTheme::createPlatformDialogHelper(DialogType dialog
         return new QCocoaColorDialogHelper();
     case QPlatformTheme::FontDialog:
         return new QCocoaFontDialogHelper();
+    case QPlatformTheme::MessageDialog:
+        return new QCocoaMessageDialog;
     default:
         return nullptr;
     }
@@ -464,9 +478,21 @@ QVariant QCocoaTheme::themeHint(ThemeHint hint) const
     return QPlatformTheme::themeHint(hint);
 }
 
-QPlatformTheme::Appearance QCocoaTheme::appearance() const
+Qt::ColorScheme QCocoaTheme::colorScheme() const
 {
-    return qt_mac_applicationIsInDarkMode() ? Appearance::Dark : Appearance::Light;
+    return m_colorScheme;
+}
+
+/*
+    Update the theme's color scheme based on the current appearance.
+
+    We can only reference the appearance on the main thread, but the
+    CoreText font engine needs to know the color scheme, and might be
+    used from secondary threads, so we cache the color scheme.
+*/
+void QCocoaTheme::updateColorScheme()
+{
+    m_colorScheme = qt_mac_applicationIsInDarkMode() ? Qt::ColorScheme::Dark : Qt::ColorScheme::Light;
 }
 
 QString QCocoaTheme::standardButtonText(int button) const
